#line 3 "REval.cweb"
#include "REvaluator.h"
#include "RtoJava.h"
#include "RStoJava.h"

SEXP R_ParseVector(SEXP, int, int *);

#line 271 "REval.cweb"
void
JavaErrorHook(SEXP call, char *message)
{
  jclass klass;
  JNIEnv *env = getJNIEnv();

#if 0
  JavaVM *vm = getJavaVM();
  (*vm)->AttachCurrentThread(vm, (void**) &env, NULL);
#endif
  
  klass = VMENV FindClass(env, "org/omegahat/R/Java/RException");

#if 1  
  VMENV ThrowNew(env, klass, message);
#else
  jmethodID mid;
  jstring msg;
  jobject exception;
  
  mid = VMENV GetMethodID(env, klass, "<init>", "(Ljava/lang/String;)V");

  msg = VMENV NewStringUTF(env, message);

  exception = VMENV NewObject(env, klass, mid, msg);
  VMENV Throw(env, exception);
#endif  
}


#line 16 "REval.cweb"
JNIEXPORT jobject JNICALL
Java_org_omegahat_R_Java_REvaluator_eval(JNIEnv *env, jobject REvaluator, jstring cmd, jboolean convert, jclass returnClass, jboolean discardResult)
{
 USER_OBJECT_ tmp, val;
 const char *str;
 jboolean isCopy;
 USER_OBJECT_ e;
 jobject ans = NULL_JAVA_OBJECT;
 int i, n;
 int status;
 char *sig;
 int errorOccurred;

 extern int R_Visible;

    RS_lockEvaluator(env, JNI_TRUE, JNI_FALSE);
 
    str = VMENV GetStringUTFChars(env, cmd, &isCopy);
    PROTECT(tmp = NEW_CHARACTER(1));
    SET_STRING_ELT(tmp, 0, COPY_TO_USER_STRING(str));
    PROTECT(e = R_ParseVector(tmp, 1, &status));
    if(isCopy == JNI_TRUE) {
       VMENV ReleaseStringUTFChars(env, cmd, str);
    }

    n = GET_LENGTH(e);
    R_Visible = FALSE;
    for(i = 0; i < n ; i++) {
      val = R_tryEval(VECTOR_ELT(e,i), R_GlobalEnv, &errorOccurred);
      if(errorOccurred) {
        UNPROTECT(2);
        RS_lockEvaluator(env, JNI_FALSE, JNI_FALSE);	
        return(NULL_JAVA_OBJECT);
      }
    }
    R_Visible = TRUE;
    PROTECT(val);
    if(discardResult == JNI_FALSE && val != NULL_USER_OBJECT) {

#ifdef LDEBUG    
  fprintf(stderr, "Converting result of eval to Java\n");
  Rf_PrintValue(val);
#endif
      ans = s_to_java_basic(env, val, NULL, &sig);
    }
    UNPROTECT(3);
    RS_lockEvaluator(env, JNI_FALSE, JNI_FALSE); 

 return(ans);
}    


#line 95 "REval.cweb"
JNIEXPORT jobject JNICALL
Java_org_omegahat_R_Java_REvaluator_call(JNIEnv *env, jobject evaluator, jstring functionName, jobjectArray args, jobjectArray argNames, jclass targetClass, jboolean convert)
{
 int numArgs = 0;
 USER_OBJECT_ fun, e, val;
 jobject result;
 int numProtect = 0;

 char *sig=NULL; 
 const char *str;
 jboolean isCopy;

 int errorOccurred;

#ifdef LDEBUG    
fprintf(stderr, "In REvaluator_call:  env = %p\n", env); fflush(stderr);
 if(args == NULL_JAVA_OBJECT) {
  fprintf(stderr, "No arguments\n"); fflush(stderr);
 }
#endif 

   if(args != NULL_JAVA_OBJECT)
     numArgs = VMENV GetArrayLength(env, args);

   RS_lockEvaluator(env, JNI_TRUE, JNI_TRUE); 


#ifdef LDEBUG    
fprintf(stderr, "Expression length %d\n", numArgs); fflush(stderr);
#endif 
   PROTECT(e = allocVector(LANGSXP, numArgs + 1));
   numProtect++;

 
   str = VMENV GetStringUTFChars(env, functionName, &isCopy);

#if 0
  /* We would like to be able to do this so that we can get more refined error messages.
     However, we cannot easily control the error handling at this point.
     So we try to eval the entire expression eventhough we could bail here if we
     knew the function wasn't there.
  */
   fun = Rf_findFun(Rf_install((char *)str),  R_GlobalEnv);
   if(isCopy == JNI_TRUE) {
       VMENV ReleaseStringUTFChars(env, functionName, str);
   }  
#else 
   fun = Rf_install(str);
#endif
   SETCAR(e, fun);

   if(numArgs > 0) {
       /* Add the ordered and named arguments to the expression. */
    int i;
    const char *keyName = NULL;
    jobject argVal, argName;
    USER_OBJECT_ tmp, cvt;
#ifdef LDEBUG       
fprintf(stderr, "Number of arguments %d\n", numArgs);fflush(stderr);
#endif
    
    tmp = CDR(e);
    for(i = 0; i < numArgs; i++) {

#ifdef LDEBUG           
fprintf(stderr, " processing arg %d)\n", i);fflush(stderr);
#endif
      argVal = VMENV GetObjectArrayElement(env, args, i);
      
#ifdef LDEBUG   
fprintf(stderr, " converting arg %d %p\n", i, argVal);fflush(stderr);
#endif
     cvt = s_from_java_basic(env, argVal, NULL, JNI_FALSE, JNI_TRUE);
     SETCAR(tmp, cvt);     
      
      argName= VMENV GetObjectArrayElement(env, argNames, i);
      if(argName == NULL_JAVA_OBJECT)
         keyName = NULL;
      else {
#ifdef LDEBUG                     
         fprintf(stderr, "getting argument name %d) %p\n", i, argName);fflush(stderr);
#endif   
         keyName = VMENV GetStringUTFChars(env, argName, &isCopy);
#ifdef LDEBUG                    
         fprintf(stderr, "argument name %d) %s\n", i, keyName);fflush(stderr);
#endif   
      }


#ifdef LDEBUG   
fprintf(stderr, " converted arg %d\n", i);fflush(stderr);
#endif

      if(keyName && keyName[0]) {
        SET_TAG(tmp, Rf_install(keyName));
        if(isCopy == JNI_TRUE) {
          VMENV ReleaseStringUTFChars(env, argName, keyName);
        }
      }
      
      tmp = CDR(tmp);
    }
  }

#ifdef LDEBUG   
fprintf(stderr, "evaluating function call\n"); fflush(stderr);
#endif 

      /* Evaluate the call to the R function.
         Ignore the return value.
       */

    val = R_tryEval(e, R_GlobalEnv, &errorOccurred);
    if(errorOccurred) {
      UNPROTECT(numProtect);
      RS_lockEvaluator(env, JNI_FALSE, JNI_TRUE);       
      return(NULL_JAVA_OBJECT);
    }
    PROTECT(val);
    numProtect++;

    if(convert == JNI_FALSE) {
#ifdef LDEBUG   
fprintf(stderr, "Setting target class to anonymous reference\n");fflush(stderr);
#endif
       /* just register this with the R reference handler and create the appropriate
	  */ 
       PROTECT(val = createRAnonymousReference(val)); numProtect++;
       result = userReferenceToJava(env, val, RS_ANONYMOUS_REFERENCE);  
    } else {
       result = s_to_java_basic(env, val, sig, &sig);
    }
  UNPROTECT(numProtect);

   RS_lockEvaluator(env, JNI_FALSE, JNI_TRUE); 
 
#ifdef LDEBUG   
fprintf(stderr, "completed native REvaluator.call()\n");fflush(stderr);
#endif 

  return(result); 
}  


#line 241 "REval.cweb"
jboolean
JNICALL 
Java_org_omegahat_Interfaces_NativeInterface_OmegaInterfaceManager_nativeSetup(
              JNIEnv *env, jclass Class, jobject This, jobject nullObj) 
{
extern void RS_JAVA_registerDefaultConverters(JNIEnv *env);
extern void R_SetErrorHook(void (*)(SEXP, char *));

  setJNIEnv(env);
  computeJavaVM(env);
  do_native_setup(This, nullObj, env);

  R_SetErrorHook(JavaErrorHook);

/*
   This is done in the Java code so no need to do it here.
   RS_JAVA_registerDefaultConverters(env);
*/


  return(JNI_TRUE);
}



